"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAllBets = getAllBets;
exports.getJokerBody = getJokerBody;
exports.getLotteryBody = getLotteryBody;
exports.getResults = getResults;
exports.getActiveBets = getActiveBets;
exports.checkIfUncheckedBets = checkIfUncheckedBets;
exports.redeliverBet = redeliverBet;

var request = _interopRequireWildcard(require("request-promise"));

var constants = _interopRequireWildcard(require("./constants"));

var responses = _interopRequireWildcard(require("./responses"));

var utils = _interopRequireWildcard(require("./utils"));

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Get all bets from /bets and push them to different arrays according to
 * properties.
 * @param {request.cookieJar} cookieJar Cookie jar
 * @returns {Object} Object with three arrays
 */
function getAllBets(_x) {
  return _getAllBets.apply(this, arguments);
}
/**
 * Generate the body for requests to buy joker.
 *
 * @param {number} gameId
 * @return {Object} Body for bet request
 */


function _getAllBets() {
  _getAllBets = _asyncToGenerator(function* (cookieJar) {
    // Options for get request.
    const options = {
      url: constants.API_URLS.BETS,
      json: true,
      jar: cookieJar
    }; // Set date to 7 days ago to only get bets within the last week.

    const oneWeekAgo = new Date();
    oneWeekAgo.setDate(oneWeekAgo.getDate() - 7); // Initialize empty arrays to be filled.

    let finishedBets = [];
    let uncheckedBets = [];
    let activeBets = [];

    try {
      const response = yield request.get(options); // Go through all bets if there are any.

      if (!_lodash.default.isEmpty(response.bets)) {
        _lodash.default.forEach(response.bets, bet => {
          const drawDate = new Date(bet.draws.draw[0].drawDate); // Check if the bet is from less than a week ago.

          if (drawDate > oneWeekAgo) {
            if (bet.couponActive) activeBets.push(bet);
            if (!bet.couponActive) finishedBets.push(bet);
            if (!bet.couponActive && !bet.checked) uncheckedBets.push(bet);
          }
        });
      }
    } catch (error) {
      console.error('Error getting bets:', error);
    } // Return an object with all three arrays.


    return {
      active: activeBets,
      finished: finishedBets,
      unchecked: uncheckedBets
    };
  });
  return _getAllBets.apply(this, arguments);
}

function getJokerBody(gameId) {
  const body = {
    paymentInfo: {
      paymentMethod: 'PLAYERACCOUNT'
    },
    wagerItems: [{
      gameId: gameId,
      numberOfDraws: 1,
      multiplier: 1,
      couponPrice: 20,
      rowPrice: 20,
      playSystem: 1,
      playMethod: 'NORMAL_PLAY'
    }]
  };
  return body;
}
/**
 * Generate the body for requests to buy lottery.
 *
 * @param {number} gameId Type of game
 * @param {number} weeks Number of weeks
 * @param {number} couponPrice Price of the coupon
 * @param {number} rowPrice Price of a single row
 * @param {number} rows Number of rows
 * @return {Object} Body for bet request
 */


function getLotteryBody(gameId, weeks, couponPrice, rowPrice, rows) {
  const body = {
    paymentInfo: {
      paymentMethod: 'PLAYERACCOUNT'
    },
    wagerItems: [{
      gameId: gameId,
      numberOfDraws: weeks,
      // Antall uker
      multiplier: 1,
      couponPrice: couponPrice,
      rowPrice: rowPrice,
      playSystem: gameId === 18 ? 0 : 1,
      playMethod: 'NORMAL_PLAY',
      selections: generateNumbers(gameId, rows),
      numRows: rows // Antall rekker

    }]
  };
  return body;
}
/**
 * Generate numbers for buying lottery.
 *
 * @param {number} gameId
 * @param {number} numRows
 * @returns {Array} Array of generated numbers
 */


function generateNumbers(gameId, numRows) {
  let rowLength, max, viking, euro, maxExtra, extraLength; // Switch case to set the correct variables.

  switch (gameId) {
    case constants.GAME.LOTTO.ID:
      rowLength = constants.GAME.LOTTO.ROW_LENGTH;
      max = constants.GAME.LOTTO.MAX_NUMBERS;
      break;

    case 16:
      rowLength = constants.GAME.EUROJACKPOT.ROW_LENGTH;
      max = constants.GAME.EUROJACKPOT.MAX_NUMBERS;
      maxExtra = constants.GAME.EUROJACKPOT.MAX_EXTRA;
      extraLength = constants.GAME.EUROJACKPOT.EXTRA_LENGTH;
      euro = true;
      break;

    case 18:
      rowLength = constants.GAME.VIKINGLOTTO.ROW_LENGTH;
      max = constants.GAME.VIKINGLOTTO.MAX_NUMBERS;
      maxExtra = constants.GAME.VIKINGLOTTO.MAX_EXTRA;
      extraLength = constants.GAME.VIKINGLOTTO.EXTRA_LENGTH;
      viking = true;
      break;

    default:
      return;
  } // Array containing the entire coupon.


  let coupon = []; // Loop through the amount of rows specified and make a row.

  for (let i = 0; i < numRows; i++) {
    let row = []; // Generate unique numbers.

    while (row.length < rowLength) {
      let r = Math.floor(Math.random() * max) + 1;
      if (row.indexOf(r) === -1) row.push(r);
    } // Sort the array.


    row.sort((a, b) => a - b); // If the game type is Vikinglotto or Eurojackpot, the extra numbers are
    // generated and pushed to the end of the row.

    if (viking || euro) {
      let extraNumbers = [];

      while (extraNumbers.length < extraLength) {
        let r = Math.floor(Math.random() * maxExtra) + 1;
        if (extraNumbers.indexOf(r) === -1) extraNumbers.push(r);
      } // Sort the extra numbers.


      extraNumbers.sort((a, b) => a - b);
      row.push(extraNumbers.join(';'));
    } // Join the array with ';' as a delimiter and push the row to the coupon.


    coupon.push(row.join(';'));
  }

  return coupon;
}
/**
 * Find one bet matching the wanted result type. Prompt if the user wants to
 * redeliver the bet after announcing if there's a prize or not.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 * @param {string} resultType Game name of the bet
 * @param {boolean} unchecked If the function should use the unchecked array
 * @param {request.CookieJar} cookieJar Cookie jar
 */


function getResults(_x2, _x3, _x4, _x5) {
  return _getResults.apply(this, arguments);
}
/**
 * Present the user with its active bets. If resultType is not specified, the
 * number of active bets and their names are said. If the resultType is
 * specified, the draw date of the bet is said.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 * @param {string} resultType Game name of the bet
 */


function _getResults() {
  _getResults = _asyncToGenerator(function* (conv, resultType, unchecked, cookieJar) {
    const bets = unchecked ? conv.data.bets.unchecked : conv.data.bets.finished; // If there's no game type specified.

    if (_lodash.default.isEmpty(resultType)) {
      console.log('Bets:', bets); // List of names of all drawn games played past week.

      let gameListNames = []; // If not empty response, get all game names of games played.

      if (!_lodash.default.isEmpty(bets)) {
        bets.forEach(bet => gameListNames.push(bet.gameName)); // Respond with the types og games the user has played. Removing duplicate
        // names so that the user won't hear e.g. "Lotto, Lotto, Lotto".

        return conv.ask(responses.results.listAllGames(formatGameListNames(gameListNames)));
      } else {
        conv.contexts.set('want_more', 2);
        conv.ask(responses.results.notPlayedPastWeek('noe'));
        return conv.ask(responses.buy.canIHelp);
      }
    } // Deleting context 'unchecked_bet' regardless if it exists or not.


    conv.contexts.delete('unchecked_bet');

    if (resultType === 'nei') {
      conv.contexts.set('want_more', 2);
      return conv.ask(responses.buy.canIHelp);
    } else if (resultType === 'joker') {
      // Manually set the result type to first bet of either 'Joker Onsdag' or
      // 'Joker Lørdag' in the list of bets.
      const jokerBet = bets.find(bet => bet.gameName.toLowerCase().startsWith(resultType));

      if (!_lodash.default.isNil(jokerBet)) {
        resultType = jokerBet.gameName;
      }
    } // Get the bet with the specified result type.


    const bet = bets.find(bet => bet.gameName.toLowerCase() === resultType.toLowerCase()); // Add a response that the user either hasn't played the requested game type
    // or has no unchecked games of that type depending on when the function is
    // called.

    if (_lodash.default.isNil(bet)) {
      conv.contexts.set('want_more', 2);
      conv.ask(responses.results.notPlayedPastWeek(_lodash.default.capitalize(resultType)));
      return conv.ask(responses.buy.canIHelp);
    }

    if (bet.hasPrize) {
      // get winAmount
      const prizeResponse = yield getPrizeDetails(cookieJar, bet.salesId);
      conv.ask(prizeResponse);
    } else {
      conv.ask(responses.results.noPrize(_lodash.default.capitalize(bet.gameName)));
    } // Set bet to checked if it is unchecked.


    if (!bet.checked) {
      setBetToChecked(bet, cookieJar);
    } // Don't ask if the user wants to redeliver the bet if it already has a bet
    // placed on that game name.


    if (!conv.data.bets.active.some(activeBet => activeBet.gameName === bet.gameName)) {
      conv.contexts.set('redeliver', 2, {
        bet
      });
      conv.ask(responses.redeliver.prompt(bet.gameName, bet.saleAmountOre / 100));
    } else {
      conv.contexts.set('want_more', 2);
      conv.ask(responses.buy.canIHelp);
    }
  });
  return _getResults.apply(this, arguments);
}

function getActiveBets(conv, resultType) {
  const bets = conv.data.bets.active; // If there's no game type specified.

  if (_lodash.default.isEmpty(resultType)) {
    // List of names of all drawn games played past week.
    let gameListNames = []; // If not empty response, get all game names of games played.

    if (!_lodash.default.isEmpty(bets)) {
      bets.forEach(bet => gameListNames.push(bet.gameName)); // Respond with the types og games the user has played. Removing duplicate
      // names so that the user won't hear e.g. "Lotto, Lotto, Lotto".

      return conv.ask(responses.activeBets.listActiveBets(gameListNames.length, formatGameListNames(gameListNames)));
    } else {
      conv.contexts.set('want_more', 2);
      conv.ask(responses.activeBets.noActiveBets);
      return conv.ask(responses.buy.canIHelp);
    }
  }

  if (resultType === 'nei') {
    conv.contexts.set('want_more', 2);
    return conv.ask(responses.buy.canIHelp);
  } else if (resultType === 'joker') {
    // Manually set the result type to first bet of either 'Joker Onsdag' or
    // 'Joker Lørdag' in the list of bets.
    const jokerBet = bets.find(bet => bet.gameName.toLowerCase().startsWith(resultType));

    if (!_lodash.default.isNil(jokerBet)) {
      resultType = jokerBet.gameName;
    }
  } // Get the bet with the specified result type.


  const bet = bets.find(bet => bet.gameName.toLowerCase() === resultType.toLowerCase());

  if (_lodash.default.isNil(bet)) {
    conv.contexts.set('want_more', 2);
    conv.ask(responses.activeBets.noActiveBetsSpecified(_lodash.default.capitalize(resultType)));
    return conv.ask(responses.buy.canIHelp);
  }

  conv.contexts.set('want_more', 2);
  conv.ask(responses.activeBets.activeBetDetails(bet));
  return conv.ask(responses.buy.canIHelp);
}
/**
 * Check if the user has any unchecked bets and return a response. If the any of
 * the bets arrays only has one bet, set a context with resultType.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 */


function checkIfUncheckedBets(conv) {
  const {
    active,
    finished,
    unchecked
  } = conv.data.bets; // Check if any of the arrays only have one type of bet played.

  checkIfOneType(finished, 'one_game', conv);
  checkIfOneType(unchecked, 'one_game_unchecked', conv);
  checkIfOneType(active, 'one_game_active', conv); // Set context and say a different response if there's unchecked bets.

  if (!_lodash.default.isEmpty(unchecked)) {
    conv.contexts.set('unchecked_bet', 2);
    return conv.ask(responses.welcome.uncheckedBets(unchecked));
  } else {
    return conv.ask(responses.buy.promptPurchase);
  }
}
/**
 * Check if the array only has one type of bets.
 *
 * @param {Array} bets Array of bets
 * @param {String} contextName Name of the context
 * @param {DialogflowConversation} conv DialogFlow conversation object
 */


function checkIfOneType(bets, contextName, conv) {
  if (bets.length === 1 || bets.length > 1 && bets.every(bet => bets[0].gameName === bet.gameName)) {
    conv.contexts.set(contextName, 99, {
      resultType: bets[0].gameName
    });
  }
}
/**
 * Redeliver a previous bet.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 * @param {request.CookieJar} cookieJar Cookie jar
 * @param {Object} bet Bet to redeliver
 */


function redeliverBet(_x6, _x7, _x8) {
  return _redeliverBet.apply(this, arguments);
}
/**
 * Set the attribute checked of a bet to true.
 *
 * @param {Object} bet Bet to set to checked
 * @param {request.CookieJar} cookieJar Cookie jar
 */


function _redeliverBet() {
  _redeliverBet = _asyncToGenerator(function* (conv, cookieJar, bet) {
    const couponPrice = bet.saleAmountOre / 100;
    const body = {
      paymentInfo: {
        paymentMethod: 'PLAYERACCOUNT'
      },
      coupons: [{
        saleId: bet.saleId,
        gameId: bet.gameNo
      }],
      couponPrice: couponPrice
    };
    const options = {
      headers: {
        'content-type': 'application/json'
      },
      url: constants.API_URLS.BETS_REDELIVER,
      jar: cookieJar,
      json: true,
      body: body
    };

    try {
      yield request.post(options);
      console.log(`Redelivered bet with sale id ${bet.saleId}`);
      conv.ask(responses.redeliver.delivered(bet.gameName, couponPrice));
    } catch (error) {
      console.error('Error redelivering bet:', error);
      const {
        code: errorCode
      } = error.error.error;

      if (errorCode.startsWith(constants.ERRORCODES.INSUFF_FUNDS)) {
        // Not enough money on player account
        // Gets the balance on the player account
        const amount = yield utils.getBalance(cookieJar);

        if (amount !== null) {
          conv.ask(responses.error.playerAccount(amount, couponPrice));
        } else {
          conv.close(responses.error.notLoggedIn);
        }
      } else if (errorCode.startsWith(constants.ERRORCODES.RG_DENIED)) {
        // Responsible Gaming (RG) reasons
        conv.ask(responses.error.gameLimit);
      } else {
        conv.close(responses.redeliver.error);
      }
    }
  });
  return _redeliverBet.apply(this, arguments);
}

function setBetToChecked(_x9, _x10) {
  return _setBetToChecked.apply(this, arguments);
}
/**
 * Make the array of the names of games unique and join the array to a string
 * with either a comma or 'og' as separator.
 *
 * @param {Array} list Array of names of games
 */


function _setBetToChecked() {
  _setBetToChecked = _asyncToGenerator(function* (bet, cookieJar) {
    const options = {
      headers: {
        'content-type': 'application/json'
      },
      url: constants.API_URLS.BETS_ATTR,
      jar: cookieJar,
      json: true,
      body: {
        saleIds: [bet.saleId],
        checked: true
      }
    };

    try {
      yield request.post(options);
      console.log(`Set attribute checked to true for bet with id ${bet.saleId}`);
    } catch (error) {
      console.error('Error setting attribute of bet:', error);
    }
  });
  return _setBetToChecked.apply(this, arguments);
}

function formatGameListNames(list) {
  const unique = _lodash.default.uniq(list);

  let formattedString = '';
  const listLength = unique.length;

  for (let i = 0; i < listLength; i++) {
    formattedString += unique[i];
    if (i < listLength - 2) formattedString += ', ';else if (i == listLength - 2) formattedString += ' og ';
  }

  return formattedString;
}
/**
 * Returns a string with a response.
 * @param {request.cookieJar} cookieJar
 * @param {number} salesId
 * @returns {string} The response to be said
 */


function getPrizeDetails(_x11, _x12) {
  return _getPrizeDetails.apply(this, arguments);
}

function _getPrizeDetails() {
  _getPrizeDetails = _asyncToGenerator(function* (cookieJar, salesId) {
    const options = {
      url: constants.API_URLS.BETS + salesId,
      json: true,
      jar: cookieJar
    };

    try {
      const response = yield request.get(options); // Get the name of the prize group.

      const prizeGroup = response.prizes[0].prizeDetails[0].prizeGroupName; // Get the win amount in ore and make it to NOK.

      const prizeAmount = response.prizeAmountOre / 100; // Return the response with prize details.

      return responses.results.prize(prizeGroup, prizeAmount, _lodash.default.capitalize(response.gameName));
    } catch (error) {
      console.error('Error getting prize details:', error);
      return responses.error.error;
    }
  });
  return _getPrizeDetails.apply(this, arguments);
}