import * as request from 'request-promise';
import * as constants from './constants';
import * as responses from './responses';
import * as utils from './utils';
import _ from 'lodash';

/**
 * Get all bets from /bets and push them to different arrays according to
 * properties.
 * @param {request.cookieJar} cookieJar Cookie jar
 * @returns {Object} Object with three arrays
 */
export async function getAllBets(cookieJar) {
  // Options for get request.
  const options = {
    url: constants.API_URLS.BETS,
    json: true,
    jar: cookieJar
  };

  // Set date to 7 days ago to only get bets within the last week.
  const oneWeekAgo = new Date();
  oneWeekAgo.setDate(oneWeekAgo.getDate() - 7);

  // Initialize empty arrays to be filled.
  let finishedBets = [];
  let uncheckedBets = [];
  let activeBets = [];

  try {
    const response = await request.get(options);
    // Go through all bets if there are any.
    if (!_.isEmpty(response.bets)) {
      _.forEach(response.bets, bet => {
        const drawDate = new Date(bet.draws.draw[0].drawDate);
        // Check if the bet is from less than a week ago.
        if (drawDate > oneWeekAgo) {
          if (bet.couponActive) activeBets.push(bet);
          if (!bet.couponActive) finishedBets.push(bet);
          if (!bet.couponActive && !bet.checked) uncheckedBets.push(bet);
        }
      });
    }
  } catch (error) {
    console.error('Error getting bets:', error);
  }

  // Return an object with all three arrays.
  return {
    active: activeBets,
    finished: finishedBets,
    unchecked: uncheckedBets
  };
}

/**
 * Generate the body for requests to buy joker.
 *
 * @param {number} gameId
 * @return {Object} Body for bet request
 */
export function getJokerBody(gameId) {
  const body = {
    paymentInfo: {
      paymentMethod: 'PLAYERACCOUNT'
    },
    wagerItems: [
      {
        gameId: gameId,
        numberOfDraws: 1,
        multiplier: 1,
        couponPrice: 20,
        rowPrice: 20,
        playSystem: 1,
        playMethod: 'NORMAL_PLAY'
      }
    ]
  };

  return body;
}

/**
 * Generate the body for requests to buy lottery.
 *
 * @param {number} gameId Type of game
 * @param {number} weeks Number of weeks
 * @param {number} couponPrice Price of the coupon
 * @param {number} rowPrice Price of a single row
 * @param {number} rows Number of rows
 * @return {Object} Body for bet request
 */
export function getLotteryBody(gameId, weeks, couponPrice, rowPrice, rows) {
  const body = {
    paymentInfo: {
      paymentMethod: 'PLAYERACCOUNT'
    },
    wagerItems: [
      {
        gameId: gameId,
        numberOfDraws: weeks, // Antall uker
        multiplier: 1,
        couponPrice: couponPrice,
        rowPrice: rowPrice,
        playSystem: gameId === 18 ? 0 : 1,
        playMethod: 'NORMAL_PLAY',
        selections: generateNumbers(gameId, rows),
        numRows: rows // Antall rekker
      }
    ]
  };

  return body;
}

/**
 * Generate numbers for buying lottery.
 *
 * @param {number} gameId
 * @param {number} numRows
 * @returns {Array} Array of generated numbers
 */
function generateNumbers(gameId, numRows) {
  let rowLength, max, viking, euro, maxExtra, extraLength;

  // Switch case to set the correct variables.
  switch (gameId) {
  case constants.GAME.LOTTO.ID:
    rowLength = constants.GAME.LOTTO.ROW_LENGTH;
    max = constants.GAME.LOTTO.MAX_NUMBERS;
    break;
  case 16:
    rowLength = constants.GAME.EUROJACKPOT.ROW_LENGTH;
    max = constants.GAME.EUROJACKPOT.MAX_NUMBERS;
    maxExtra = constants.GAME.EUROJACKPOT.MAX_EXTRA;
    extraLength = constants.GAME.EUROJACKPOT.EXTRA_LENGTH;
    euro = true;
    break;
  case 18:
    rowLength = constants.GAME.VIKINGLOTTO.ROW_LENGTH;
    max = constants.GAME.VIKINGLOTTO.MAX_NUMBERS;
    maxExtra = constants.GAME.VIKINGLOTTO.MAX_EXTRA;
    extraLength = constants.GAME.VIKINGLOTTO.EXTRA_LENGTH;
    viking = true;
    break;
  default:
    return;
  }

  // Array containing the entire coupon.
  let coupon = [];

  // Loop through the amount of rows specified and make a row.
  for (let i = 0; i < numRows; i++) {
    let row = [];

    // Generate unique numbers.
    while (row.length < rowLength) {
      let r = Math.floor(Math.random() * max) + 1;
      if (row.indexOf(r) === -1) row.push(r);
    }

    // Sort the array.
    row.sort((a, b) => a - b);

    // If the game type is Vikinglotto or Eurojackpot, the extra numbers are
    // generated and pushed to the end of the row.
    if (viking || euro) {
      let extraNumbers = [];
      while (extraNumbers.length < extraLength) {
        let r = Math.floor(Math.random() * maxExtra) + 1;
        if (extraNumbers.indexOf(r) === -1) extraNumbers.push(r);
      }

      // Sort the extra numbers.
      extraNumbers.sort((a, b) => a - b);

      row.push(extraNumbers.join(';'));
    }
    // Join the array with ';' as a delimiter and push the row to the coupon.
    coupon.push(row.join(';'));
  }

  return coupon;
}

/**
 * Find one bet matching the wanted result type. Prompt if the user wants to
 * redeliver the bet after announcing if there's a prize or not.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 * @param {string} resultType Game name of the bet
 * @param {boolean} unchecked If the function should use the unchecked array
 * @param {request.CookieJar} cookieJar Cookie jar
 */
export async function getResults(conv, resultType, unchecked, cookieJar) {
  const bets = unchecked ? conv.data.bets.unchecked : conv.data.bets.finished;

  // If there's no game type specified.
  if (_.isEmpty(resultType)) {
    console.log('Bets:', bets);
    // List of names of all drawn games played past week.
    let gameListNames = [];
    // If not empty response, get all game names of games played.
    if (!_.isEmpty(bets)) {
      bets.forEach(bet => gameListNames.push(bet.gameName));

      // Respond with the types og games the user has played. Removing duplicate
      // names so that the user won't hear e.g. "Lotto, Lotto, Lotto".
      return conv.ask(
        responses.results.listAllGames(formatGameListNames(gameListNames))
      );
    } else {
      conv.contexts.set('want_more', 2);
      conv.ask(responses.results.notPlayedPastWeek('noe'));
      return conv.ask(responses.buy.canIHelp);
    }
  }
  
  // Deleting context 'unchecked_bet' regardless if it exists or not.
  conv.contexts.delete('unchecked_bet');

  if (resultType === 'nei') {
    conv.contexts.set('want_more', 2);
    return conv.ask(responses.buy.canIHelp);
  } else if (resultType === 'joker') {
    // Manually set the result type to first bet of either 'Joker Onsdag' or
    // 'Joker Lørdag' in the list of bets.

    const jokerBet = bets.find(bet =>
      bet.gameName.toLowerCase().startsWith(resultType)
    );

    if (!_.isNil(jokerBet)) {
      resultType = jokerBet.gameName;
    }
  }

  // Get the bet with the specified result type.
  const bet = bets.find(
    bet => bet.gameName.toLowerCase() === resultType.toLowerCase()
  );

  // Add a response that the user either hasn't played the requested game type
  // or has no unchecked games of that type depending on when the function is
  // called.
  if (_.isNil(bet)) {
    conv.contexts.set('want_more', 2);
    conv.ask(responses.results.notPlayedPastWeek(_.capitalize(resultType)));
    return conv.ask(responses.buy.canIHelp);
  }

  if (bet.hasPrize) {
    // get winAmount
    const prizeResponse = await getPrizeDetails(cookieJar, bet.salesId);

    conv.ask(prizeResponse);
  } else {
    conv.ask(responses.results.noPrize(_.capitalize(bet.gameName)));
  }

  // Set bet to checked if it is unchecked.
  if (!bet.checked) {
    setBetToChecked(bet, cookieJar);
  }


  // Don't ask if the user wants to redeliver the bet if it already has a bet
  // placed on that game name.
  if (
    !conv.data.bets.active.some(
      activeBet => activeBet.gameName === bet.gameName
    )
  ) {
    conv.contexts.set('redeliver', 2, { bet });

    conv.ask(responses.redeliver.prompt(bet.gameName, bet.saleAmountOre / 100));
  } else {
    conv.contexts.set('want_more', 2);
    conv.ask(responses.buy.canIHelp);
  }
}

/**
 * Present the user with its active bets. If resultType is not specified, the
 * number of active bets and their names are said. If the resultType is
 * specified, the draw date of the bet is said.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 * @param {string} resultType Game name of the bet
 */
export function getActiveBets(conv, resultType) {
  const bets = conv.data.bets.active;
  // If there's no game type specified.
  if (_.isEmpty(resultType)) {
    // List of names of all drawn games played past week.
    let gameListNames = [];
    // If not empty response, get all game names of games played.
    if (!_.isEmpty(bets)) {
      bets.forEach(bet => gameListNames.push(bet.gameName));

      // Respond with the types og games the user has played. Removing duplicate
      // names so that the user won't hear e.g. "Lotto, Lotto, Lotto".
      return conv.ask(
        responses.activeBets.listActiveBets(
          gameListNames.length,
          formatGameListNames(gameListNames)
        )
      );
    } else {
      conv.contexts.set('want_more', 2);
      conv.ask(responses.activeBets.noActiveBets);
      return conv.ask(responses.buy.canIHelp);
    }
  }

  if (resultType === 'nei') {
    conv.contexts.set('want_more', 2);
    return conv.ask(responses.buy.canIHelp);
  } else if (resultType === 'joker') {
    // Manually set the result type to first bet of either 'Joker Onsdag' or
    // 'Joker Lørdag' in the list of bets.
    const jokerBet = bets.find(bet =>
      bet.gameName.toLowerCase().startsWith(resultType)
    );

    if (!_.isNil(jokerBet)) {
      resultType = jokerBet.gameName;
    }
  }

  // Get the bet with the specified result type.
  const bet = bets.find(
    bet => bet.gameName.toLowerCase() === resultType.toLowerCase()
  );

  if (_.isNil(bet)) {
    conv.contexts.set('want_more', 2);
    conv.ask(
      responses.activeBets.noActiveBetsSpecified(_.capitalize(resultType))
    );
    return conv.ask(responses.buy.canIHelp);
  }

  conv.contexts.set('want_more', 2);
  conv.ask(responses.activeBets.activeBetDetails(bet));
  return conv.ask(responses.buy.canIHelp);
}

/**
 * Check if the user has any unchecked bets and return a response. If the any of
 * the bets arrays only has one bet, set a context with resultType.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 */
export function checkIfUncheckedBets(conv) {
  const { active, finished, unchecked } = conv.data.bets;

  // Check if any of the arrays only have one type of bet played.
  checkIfOneType(finished, 'one_game', conv);
  checkIfOneType(unchecked, 'one_game_unchecked', conv);
  checkIfOneType(active, 'one_game_active', conv);

  // Set context and say a different response if there's unchecked bets.
  if (!_.isEmpty(unchecked)) {
    conv.contexts.set('unchecked_bet', 2);
    return conv.ask(responses.welcome.uncheckedBets(unchecked));
  } else {
    return conv.ask(responses.buy.promptPurchase);
  }
}

/**
 * Check if the array only has one type of bets.
 *
 * @param {Array} bets Array of bets
 * @param {String} contextName Name of the context
 * @param {DialogflowConversation} conv DialogFlow conversation object
 */
function checkIfOneType(bets, contextName, conv) {
  if (
    bets.length === 1 ||
    (bets.length > 1 && bets.every(bet => bets[0].gameName === bet.gameName))
  ) {
    conv.contexts.set(contextName, 99, {
      resultType: bets[0].gameName
    });
  }
}

/**
 * Redeliver a previous bet.
 *
 * @param {DialogflowConversation} conv DialogFlow conversation object
 * @param {request.CookieJar} cookieJar Cookie jar
 * @param {Object} bet Bet to redeliver
 */
export async function redeliverBet(conv, cookieJar, bet) {
  const couponPrice = bet.saleAmountOre / 100;

  const body = {
    paymentInfo: {
      paymentMethod: 'PLAYERACCOUNT'
    },
    coupons: [
      {
        saleId: bet.saleId,
        gameId: bet.gameNo
      }
    ],
    couponPrice: couponPrice
  };

  const options = {
    headers: { 'content-type': 'application/json' },
    url: constants.API_URLS.BETS_REDELIVER,
    jar: cookieJar,
    json: true,
    body: body
  };

  try {
    await request.post(options);
    console.log(`Redelivered bet with sale id ${bet.saleId}`);
    conv.ask(responses.redeliver.delivered(bet.gameName, couponPrice));
  } catch (error) {
    console.error('Error redelivering bet:', error);
    const { code: errorCode } = error.error.error;

    if (errorCode.startsWith(constants.ERRORCODES.INSUFF_FUNDS)) {
      // Not enough money on player account
      // Gets the balance on the player account
      const amount = await utils.getBalance(cookieJar);

      if (amount !== null) {
        conv.ask(responses.error.playerAccount(amount, couponPrice));
      } else {
        conv.close(responses.error.notLoggedIn);
      }
    } else if (errorCode.startsWith(constants.ERRORCODES.RG_DENIED)) {
      // Responsible Gaming (RG) reasons
      conv.ask(responses.error.gameLimit);
    } else {
      conv.close(responses.redeliver.error);
    }
  }
}

/**
 * Set the attribute checked of a bet to true.
 *
 * @param {Object} bet Bet to set to checked
 * @param {request.CookieJar} cookieJar Cookie jar
 */
async function setBetToChecked(bet, cookieJar) {
  const options = {
    headers: { 'content-type': 'application/json' },
    url: constants.API_URLS.BETS_ATTR,
    jar: cookieJar,
    json: true,
    body: { saleIds: [bet.saleId], checked: true }
  };

  try {
    await request.post(options);
    console.log(`Set attribute checked to true for bet with id ${bet.saleId}`);
  } catch (error) {
    console.error('Error setting attribute of bet:', error);
  }
}

/**
 * Make the array of the names of games unique and join the array to a string
 * with either a comma or 'og' as separator.
 *
 * @param {Array} list Array of names of games
 */
function formatGameListNames(list) {
  const unique = _.uniq(list);

  let formattedString = '';
  const listLength = unique.length;

  for (let i = 0; i < listLength; i++) {
    formattedString += unique[i];
    if (i < listLength - 2) formattedString += ', ';
    else if (i == listLength - 2) formattedString += ' og ';
  }

  return formattedString;
}

/**
 * Returns a string with a response.
 * @param {request.cookieJar} cookieJar
 * @param {number} salesId
 * @returns {string} The response to be said
 */
async function getPrizeDetails(cookieJar, salesId) {
  const options = {
    url: constants.API_URLS.BETS + salesId,
    json: true,
    jar: cookieJar
  };

  try {
    const response = await request.get(options);

    // Get the name of the prize group.
    const prizeGroup = response.prizes[0].prizeDetails[0].prizeGroupName;
    // Get the win amount in ore and make it to NOK.
    const prizeAmount = response.prizeAmountOre / 100;

    // Return the response with prize details.
    return responses.results.prize(
      prizeGroup,
      prizeAmount,
      _.capitalize(response.gameName)
    );
  } catch (error) {
    console.error('Error getting prize details:', error);
    return responses.error.error;
  }
}
