import { dialogflow, Confirmation, NewSurface } from 'actions-on-google';
import * as functions from 'firebase-functions';
import * as admin from 'firebase-admin';
import * as request from 'request-promise';
import requestDebug from 'request-debug';
import dashbot from 'dashbot';
import _ from 'lodash';

import * as responses from './responses';
import * as constants from './constants';
import * as utils from './utils';
import * as bets from './bets';
import * as login from './login';
import * as notifications from './notifications';

// Log the requests
requestDebug(request);

// Cookie jar to save cookies and use them for subsequent requests.
let cookieJar = request.jar();

// Initializing Firestore
admin.initializeApp();

// Reference to the database
const db = admin.firestore();

// Initializing the DialogFlow application
const app = dialogflow({
  debug: false
});

// Pass a dialogflow object to dashbot
dashbot(functions.config().dashbot.key).google.configHandler(app);

// Handle the Dialogflow intent named 'Default Welcome Intent'.
app.intent('Default Welcome Intent', async conv => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  // Create a new jar to start with it empty. Exprienced issues with the cookie
  // jar persisting through sessions and wrong being logged in as a different
  // user.
  cookieJar = request.jar();

  console.log('User storage:', conv.user.storage);
  const hasScreen = conv.surface.capabilities.has(constants.CAPABILITY);
  const screenAvailable = conv.available.surfaces.capabilities.has(
    constants.CAPABILITY
  );

  if (conv.user.storage.db) {
    // Get the document from the database.
    const doc = await db
      .collection('users')
      .doc(conv.user.storage.db)
      .get();

    let sessionID, savedFnr;

    // Check if the doc exists or not.
    if (doc.exists) {
      const data = doc.data();
      // Destructure the data object.
      ({ token: sessionID, sms: conv.data.sms } = data);
      // Decrypt the birth number if it is saved.
      if (!_.isNil(data.fnr)) {
        savedFnr = utils.decrypt(data.fnr);
      }
      console.log('Fetched user from database:', data);
    } else {
      console.log(
        'Document with id %s did not exist in database',
        conv.user.storage.db
      );
      // Clear the storage if the document doesn't exist.
      conv.user.storage = {};
    }

    if (!_.isNil(sessionID)) {
      cookieJar.setCookie(sessionID, constants.COOKIE_HOST);
    }

    // Get user's name
    conv.data.name = await utils.getName(cookieJar);

    console.log('Name: ', conv.data.name);

    // If the name is set, the user's token is valid.
    if (conv.data.name) {
      conv.contexts.set('token_valid', 99);

      conv.ask(responses.welcome.welcomeToNTWithName(conv.data.name));
      // Get user's bets
      conv.data.bets = await bets.getAllBets(cookieJar);
      console.log('Bets: ', JSON.stringify(conv.data.bets, null, 2));

      // Check if the user has any unchecked bets and announce if true.
      return bets.checkIfUncheckedBets(conv);
    } else {
      // Check if the birth number is saved.
      if (savedFnr) {
        // Sending API request to receive a one-time password
        return login.sendPostFnr(savedFnr, cookieJar).then(response => {
          const sessionID = utils.getCookie(response);
          cookieJar.setCookie(sessionID, constants.COOKIE_HOST);

          // Set context with sessionID and savedFnr as parameters.
          conv.contexts.set('token_invalid_fnr', 99, { sessionID, savedFnr });

          // Initialize the counter for max limit on wrong OTP.
          conv.data.otpCounter = 0;
          conv.ask(responses.welcome.welcomeBackNoSound);
          conv.ask(responses.otpfnr.receivingAndSayingOtp);
        });
      } else {
        conv.ask(responses.welcome.welcomeNoSound);

        // Check if the user is on a device with screen or not.
        if (!hasScreen && screenAvailable) {
          // Ask if the user wants to send the conversation to new surface.
          return conv.ask(
            new NewSurface({
              context: responses.login.revalidateSurface,
              notification: constants.NEW_SURFACE.NOTIFICATION,
              capabilities: constants.NEW_SURFACE.CAPABILITIES
            })
          );
        } else if (hasScreen) {
          conv.contexts.set('token_invalid', 99);
          return conv.ask(responses.login.revalidateScreen);
        } else {
          return conv.close(responses.login.revalidateNoScreen);
        }
      }
    }
  } else {
    conv.ask(responses.welcome.welcomeNoSound);

    // Check if the user is on a device with screen or not.
    if (!hasScreen && screenAvailable) {
      // Ask if the user wants to send the conversation to new surface.
      return conv.ask(
        new NewSurface({
          context: responses.login.loginSurface,
          notification: constants.NEW_SURFACE.NOTIFICATION,
          capabilities: constants.NEW_SURFACE.CAPABILITIES
        })
      );
    } else if (hasScreen) {
      conv.contexts.set('token_invalid', 99);
      conv.ask(responses.login.loginScreen);
    } else {
      conv.close(responses.login.loginNoScreen);
    }
  }
});

// Handle the intent named 'newSurface'
app.intent('newSurface', (conv, params, newSurface) => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;
  // Check if the user said yes to send conversation to new surface.
  if (newSurface.status === 'OK') {
    conv.contexts.set('token_invalid', 99);
    conv.ask(responses.login.loginScreen);
  } else {
    conv.close(responses.login.loginNoScreen);
  }
});

// Handle the intent named 'postFnr'
//// Input contexts: token_invalid
//// Output contexts: token_invalid_fnr (fra fulfillment)
//// Parameters: fnr
app.intent('postFnr', (conv, { fnr }) => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  const validatedFnr = login.validateFnr(fnr);

  if (validatedFnr === null) {
    return conv.ask(responses.otpfnr.invalidFnr);
  }
  // POST request to receive an SMS
  return login
    .sendPostFnr(validatedFnr, cookieJar)
    .then(response => {
      const sessionID = utils.getCookie(response);
      cookieJar.setCookie(sessionID, constants.COOKIE_HOST);

      conv.contexts.delete('token_invalid');

      // Set new context with parameters.
      conv.contexts.set('token_invalid_fnr', 99, {
        fnr: validatedFnr,
        sessionID
      });

      // Initialize the counter for max limit on wrong OTP.
      conv.data.otpCounter = 0;

      return conv.ask(responses.otpfnr.receivingAndWritingOtp);
    })
    .catch(error => {
      console.error('Error with the request:', error);
    });
});

// Handle the intent named 'postOTP'
//// Input contexts: token_invalid_fnr
//// Output contexts: save_data (fra webhook)
//// Parameters: otp
app.intent('postOTP', async (conv, { otp }) => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  // Get parameters from saved context.
  const { fnr, sessionID, savedFnr } = conv.contexts.get(
    'token_invalid_fnr'
  ).parameters;

  const validatedOTP = login.validateOTP(otp);
  // Check if the user entered an invalid one-time password.
  if (validatedOTP === null) {
    // Check if the limit for entering OTP is reached.
    if (conv.data.otpCounter++ > 3) {
      return conv.close(responses.welcome.maxInvalidOTP);
    }

    return conv.ask(responses.otpfnr.invalidOTP);
  }

  const options = {
    headers: { 'content-type': 'application/json' },
    url: constants.API_URLS.LOGIN + validatedOTP,
    jar: cookieJar
  };

  try {
    const response = await request.post(options);
    const parsedResp = JSON.parse(response);
    if (parsedResp.data.result.statusCode === 2)
      throw new Error('Incorrect one-time password');
    // Set new context with parameters.
    conv.contexts.delete('token_invalid_fnr');

    // Check if the user's birth number is saved in the database.
    if (savedFnr) {
      const data = {
        fnr: utils.encrypt(savedFnr),
        token: sessionID
      };

      try {
        await login.saveDocumentToDB(db, conv, data);
        // Get user's name
        conv.data.name = await utils.getName(cookieJar);
        // Get user's bets
        conv.data.bets = await bets.getAllBets(cookieJar);
        console.log('Bets:', JSON.stringify(conv.data.bets, null, 2));

        // Set context to 'token_valid'.
        conv.contexts.set('token_valid', 99);
        conv.ask(responses.login.connected(conv.data.name));

        // Check if the user has any unchecked bets and announce if true.
        return bets.checkIfUncheckedBets(conv);
      } catch (error) {
        console.error('Error saving the document:', error);
      }
    } else {
      conv.contexts.set('save_data', 1, { fnr, sessionID });
      // Ask the user if it wants to save the birth number to the database.
      return conv.ask(
        new Confirmation(responses.login.connectedAskForConfirmationToSave)
      );
    }
  } catch (error) {
    console.error(error);
    // Check if the limit for entering OTP is reached.
    if (conv.data.otpCounter++ > 3) {
      return conv.close(responses.welcome.maxInvalidOTP);
    }
    return conv.ask(responses.otpfnr.wrongOTP);
  }
});

// Handle the intent named 'saveToDatabase'
//// Input contexts: save_data
//// Output contexts:
//// Parameters: confirmation
//// Confirmation: https://developers.google.com/actions/assistant/helpers#confirmation
app.intent('saveToDatabase', async (conv, params, confirmation) => {
  // Get the parameters from the saved context.
  const { fnr, sessionID } = conv.contexts.get('save_data').parameters;
  let data;

  // Check if the user says "yes" or "no". If "yes", the fnr is added to the
  // data that will be saved in the database.
  if (confirmation) {
    data = {
      fnr: utils.encrypt(fnr),
      token: sessionID
    };
  } else {
    data = {
      token: sessionID
    };
  }

  // Check if the user already as a reference to a document. Updates the
  // document if it has one, adds a new one if not.
  if (conv.user.storage.db) {
    login
      .saveDocumentToDB(db, conv, data)
      .then(doc => console.log('Saved document to database:', doc))
      .catch(error => console.error('Error adding document:', error));
  } else {
    conv.user.storage.db = await db
      .collection('users')
      .add(data)
      .then(docRef => {
        console.log('Saved document to database:', docRef.id);
        return docRef.id;
      })
      .catch(error => console.error('Error adding document:', error));
  }

  // Get user's name
  conv.data.name = await utils.getName(cookieJar);
  // Get user's bets
  conv.data.bets = await bets.getAllBets(cookieJar);
  console.log('Bets: ', JSON.stringify(conv.data.bets, null, 2));

  conv.contexts.set('token_valid', 99);
  conv.ask(responses.login.connected(conv.data.name));

  // Check if the user has any unchecked bets and announce if true.
  return bets.checkIfUncheckedBets(conv);
});

// Handle the intent named 'buyLottery'
//// Input contexts: token_valid
//// Output contexts: confirm_purchase | ask_onemore_row (last only if viking(lotto) and one row)
//// Parameters: lottery_type, weeks, rows
app.intent('buyLottery', (conv, { lottery_type, weeks, rows }) => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;
  // set the gameID, coupon price and row price for each individual game
  let gameId, couponPrice, rowPrice;

  const hasType = lottery_type.length > 0;
  const hasWeeks = weeks.length > 0;
  const hasRows = rows.length > 0;

  // Checks what params that's missing and ask accordingly
  if (!hasType) {
    return conv.ask(responses.buyLotto.selectType);
  } else if (!hasWeeks) {
    return conv.ask(responses.buyLotto.selectWeeks);
  } else if (!hasRows) {
    if (lottery_type === 'eurojackpot') {
      return conv.ask(responses.buyLotto.selectRowsEuro);
    } else {
      return conv.ask(responses.buyLotto.selectRowsNotEuro(lottery_type));
    }
  }

  // set JSON body values depending on actual game
  switch (lottery_type) {
  case constants.GAME.LOTTO.NAME:
    gameId = constants.GAME.LOTTO.ID;
    rowPrice = constants.GAME.LOTTO.ROW_PRICE;
    break;
  case constants.GAME.VIKINGLOTTO.NAME:
    gameId = constants.GAME.VIKINGLOTTO.ID;
    rowPrice = constants.GAME.VIKINGLOTTO.ROW_PRICE;
    break;
  case constants.GAME.EUROJACKPOT.NAME:
    gameId = constants.GAME.EUROJACKPOT.ID;
    rowPrice = constants.GAME.EUROJACKPOT.ROW_PRICE;
    break;
  default:
    console.error('Value error for lottery type');
    return conv.close(responses.error.error);
  }

  if (lottery_type !== constants.GAME.EUROJACKPOT.NAME && rows < 2) {
    rows = 2;
    couponPrice = rowPrice * rows * weeks;

    // Set ask_onemore_row for asking about two rows instead of one
    conv.contexts.set('ask_onemore_row', 4, {
      lottery_type,
      rows,
      weeks,
      couponPrice,
      gameId,
      rowPrice
    });
    conv.ask(responses.buyLotto.askOneMore(lottery_type));
  } else {
    couponPrice = rowPrice * rows * weeks;
    // JSON body for post request in confirmPurchase intent
    const body = bets.getLotteryBody(
      gameId,
      weeks,
      couponPrice,
      rowPrice,
      rows
    );

    // Set context to make JSON body available in confirmPurchase intent
    conv.contexts.set('confirm_purchase', 4, { body, couponPrice });
    conv.ask(
      new Confirmation(
        responses.buyLotto.repeatChoice(lottery_type, rows, weeks, couponPrice)
      )
    );
  }
});

// Handle the intent named 'buyJoker'
//// Input contexts: token_valid
//// Output contexts: confirm_purchase
//// Parameters: day
app.intent('buyJoker', (conv, { day }) => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;
  // gameId for Wednesday or Saturday
  let gameId;
  // Set the couponPrice for Joker
  const couponPrice = 20;
  // Set the gameId for the JSON payload
  if (day === 'lørdag') {
    gameId = 6;
  } else if (day === 'onsdag') {
    gameId = 7;
  } else {
    // If day doesn't match "onsdag" or "lørdag"
    console.error('Value need to be either wednesday or saturday');
    return conv.close(responses.error.error);
  }
  const body = bets.getJokerBody(gameId);

  conv.contexts.set('confirm_purchase', 4, {
    body,
    couponPrice
  });
  conv.ask(new Confirmation(responses.buyJoker.repeatChoice(day, couponPrice)));
});

// Handle the intent named 'confirmPurchase'
//// Input contexts: confirm_purchase, token_valid
//// Output contexts:
//// Parameters: confirmation
app.intent('confirmPurchase', async (conv, params, confirmation) => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  // If user did not give permission to purchase item
  if (!confirmation) {
    conv.contexts.delete('confirm_purchase');
    conv.contexts.set('want_more', 2);
    return conv.ask(responses.buy.buySomethingElse);
  }

  // Get the JSON body and couponPrice from confirm_purchase context
  const { body, couponPrice } = conv.contexts.get(
    'confirm_purchase'
  ).parameters;

  // Get user's phone number for sending SMS.
  const phoneNumber = await utils.getPhone(cookieJar);

  // Create JSON payload
  const options = {
    url: constants.API_URLS.BETS,
    body: body,
    json: true,
    jar: cookieJar
  };

  // Send a POST request to the external API
  return request
    .post(options)
    .then(response => {
      console.log(
        'Response after purchase:',
        JSON.stringify(response, null, 2)
      );
      conv.contexts.delete('confirm_purchase');

      // Get the information from the first receipt in the array of receipts,
      // since we only get one receipt from a purchase.
      const { gameName, couponPrice, id, lastDrawUTC } = response.receipts[0];

      // Create body for the SMS.
      const smsBody = notifications.createSMS(
        gameName,
        couponPrice,
        id,
        lastDrawUTC
      );

      // Send SMS with receipt then log.
      notifications
        .sendSMS(smsBody, phoneNumber)
        .then(() =>
          console.log(`SMS with receipt (id: ${id}) sent to ${phoneNumber}.`)
        )
        .catch(error => console.error('Error sending SMS:', error));

      console.log(
        'User with document ID %s bought %s for %d NOK',
        conv.user.storage.db,
        gameName,
        couponPrice
      );

      conv.ask(responses.confirm.confirmationSMS(conv.data.name));

      // Check if the user has said yes or no to receive SMS notifications.
      if (conv.data.sms == null) {
        conv.contexts.set('want_sms_notification', 2, {
          phoneNumber,
          lastDrawUTC,
          gameName,
          id
        });

        return conv.ask(responses.confirm.smsNotification);
      } else if (conv.data.sms) {
        // Add the user to the 'notifications' collection in Firestore.
        notifications.saveNotificationToDB(
          db,
          utils.encrypt(phoneNumber),
          lastDrawUTC,
          gameName,
          id
        );
      }

      conv.contexts.set('want_more', 2);
      conv.ask(responses.buy.canIHelp);
    })
    .catch(async error => {
      console.error('Error completing the purchase:', error);
      // Get the specific error code
      const { code: errorCode } = error.error.error;

      // Error code scenarios
      // Not logged in
      if (errorCode.startsWith(constants.ERRORCODES.NOT_LOGGED_IN)) {
        conv.close(responses.error.notLoggedIn);
      } else if (errorCode.startsWith(constants.ERRORCODES.INSUFF_FUNDS)) {
        // Not enough money on player account
        // Gets the balance on the player account
        const amount = await utils.getBalance(cookieJar);

        if (amount !== null) {
          conv.contexts.set('want_more', 2);
          conv.ask(responses.error.playerAccount(amount, couponPrice));
          conv.ask(responses.buy.canIHelp);
        } else {
          conv.close(responses.error.notLoggedIn);
        }
      } else if (errorCode.startsWith(constants.ERRORCODES.RG_DENIED)) {
        // Responsible Gaming (RG) reasons
        conv.contexts.set('want_more', 2);
        conv.ask(responses.error.gameLimit);
        conv.ask(responses.buy.canIHelp);
      } else {
        conv.close(responses.error.somethingWentWrong);
      }
    });
});

// Handle the intent named 'wantSMSNotification - yes'
app.intent('wantSMSNotification - yes', async conv => {
  // Get params from context.
  const { phoneNumber, lastDrawUTC, gameName, id } = conv.contexts.get(
    'want_sms_notification'
  ).parameters;

  // Add the user to the 'notifications' collection in Firestore.
  notifications.saveNotificationToDB(
    db,
    utils.encrypt(phoneNumber),
    lastDrawUTC,
    gameName,
    id
  );
  // Set sms to true to indicate that user wants notifications.
  notifications.updateNotificationSetting(db, conv, true);

  conv.contexts.delete('want_sms_notification');
  conv.contexts.set('want_more', 2);

  conv.ask(responses.confirm.smsNotificationYes);
  conv.ask(responses.buy.canIHelp);
});

// Handle the intent named 'wantSMSNotification - no'
app.intent('wantSMSNotification - no', conv => {
  // Set sms to false to indicate that user doesn't want notifications.
  notifications.updateNotificationSetting(db, conv, false);

  conv.contexts.delete('want_sms_notification');
  conv.contexts.set('want_more', 2);

  conv.ask(responses.confirm.smsNotificationNo);
  conv.ask(responses.buy.canIHelp);
});

// Handle the intent named 'changeAlertSetting'
app.intent('changeAlertSetting', (conv, { smsChange }) => {
  const wantsNotifications = smsChange === 'vil';

  if (conv.data.sms && wantsNotifications) {
    conv.ask(responses.sms.alreadySending);
  } else if (conv.data.sms && !wantsNotifications) {
    // Delete all current notifications
    notifications.deleteAllNotifications(db, cookieJar);

    // Update notification setting and variable saved in conv.data to false
    notifications.updateNotificationSetting(db, conv, false);
    conv.data.sms = false;

    conv.ask(responses.sms.stopSending);
  } else if (!conv.data.sms && wantsNotifications) {
    // Add all currently active games
    notifications.addAllNotifications(db, cookieJar);

    // Update notification setting and variable saved in conv.data to true
    notifications.updateNotificationSetting(db, conv, true);
    conv.data.sms = true;

    conv.ask(responses.sms.startSending);
  } else {
    conv.ask(responses.sms.notSending);
  }

  conv.ask(responses.buy.canIHelp);
});

// Handle the intent named 'utils.getBalance'
//// Input contexts: token_invalid
app.intent('getBalance', async conv => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  const amount = await utils.getBalance(cookieJar);

  if (amount !== null) {
    conv.contexts.set('want_more', 2);
    conv.ask(responses.balance.success(amount));
    conv.ask(responses.buy.canIHelp);
  } else {
    conv.close(responses.error.notLoggedIn);
  }
});

app.intent('goodbye', async conv => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;
  // Update token i database.
  await login.updateTokenOnClose(db, conv, cookieJar);

  const name = conv.data.name ? conv.data.name : '';
  conv.close(responses.bye.goodbye(name));
});

app.intent('clearStorage', async conv => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  // Gets a document if the user has a reference
  if (conv.user.storage.db) {
    const docRef = await db.collection('users').doc(conv.user.storage.db);
    const doc = await docRef.get();

    // Deletes the document if it exists
    if (doc.exists) {
      await docRef
        .delete()
        .then(console.log(`Deleted user with ID ${conv.user.storage.db}`))
        .catch(error =>
          console.error(
            `Error deleting user with ID ${conv.user.storage.db}`,
            error
          )
        );
    } else {
      // If a reference exists but its document doesn't log it as an error
      console.error(
        `Tried to delete user with ref ${
          conv.user.storage.db
        } but it doesn't exists.`
      );
    }
  }

  // Deletes the reference to the database document
  conv.user.storage = {};

  conv.ask(responses.clearData.deleted);
  conv.close(responses.bye.goodbye(''));
});

// Triggered when the user says yes to playing with two rows instead of one in lotto and vikinglotto
// Input context: ask_onemore_row
app.intent('buyOneMore - yes', conv => {
  // Reset fallbackCount
  conv.data.fallbackCount = 0;

  const {
    lottery_type,
    rows,
    weeks,
    couponPrice,
    gameId,
    rowPrice
  } = conv.contexts.get('ask_onemore_row').parameters;
  conv.contexts.delete('ask_onemore_row');

  // JSON body for post request in confirmPurchase intent
  const body = bets.getLotteryBody(gameId, weeks, couponPrice, rowPrice, rows);

  // Set context to make JSON body available in confirmPurchase intent
  conv.contexts.set('confirm_purchase', 4, { body, couponPrice });
  conv.ask(
    new Confirmation(
      responses.buyLotto.repeatChoice(lottery_type, rows, weeks, couponPrice)
    )
  );
});

// General intent triggered when the user answers no to "do you want more?"
// Input context: want_more
app.intent('wantMore - no', async conv => {
  // Update token i database.
  await login.updateTokenOnClose(db, conv, cookieJar);

  const name = conv.data.name ? conv.data.name : '';
  return conv.close(responses.bye.acknowledgeAndBye(name));
});

// Gives result to the user for one or more bets played and drawn, but
// unchecked, in the past week
app.intent('announceUncheckedBet', async (conv, { resultType }) => {
  await bets.getResults(conv, resultType, true, cookieJar);
});

// Gives result to the user for one or more bets played and drawn in the past
// week
app.intent('getResult', async (conv, { resultType }) => {
  await bets.getResults(conv, resultType, false, cookieJar);
});

// Redelivers a bet.
app.intent('redeliverBet - yes', async conv => {
  const { bet } = conv.contexts.get('redeliver').parameters;

  await bets.redeliverBet(conv, cookieJar, bet);

  conv.contexts.delete('redeliver');
  conv.contexts.set('want_more', 2);
  conv.ask(responses.redeliver.more);
});

// Triggered when the user says "hjelp" or similar
// Reponds with relevalt information based on set contexts
// Sets the contexts anew
app.intent('infoGeneral', conv => {
  let context;
  if ((context = conv.contexts.get('ask_onemore_row'))) {
    // Set context and check if null
    conv.ask(responses.help.oneMoreRow(context.parameters.lottery_type));
    conv.contexts.set('ask_onemore_row', 2, context.parameters); // Renew the context
  } else if ((context = conv.contexts.get('want_sms_notification'))) {
    // Ikke testet
    conv.ask(responses.help.wantSmsNotification);
    conv.contexts.set('want_sms_notification', 2, context.parameters); // Renew the context
  } else if ((context = conv.contexts.get('redeliver'))) {
    // Get context parameters from redeliver
    const { bet } = conv.contexts.get('redeliver').parameters;
    conv.ask(responses.help.redeliver(bet.saleAmountOre / 100, bet.gameName));
    conv.contexts.set('redeliver', 2, context.parameters); // Renew the context
  } else if ((context = conv.contexts.get('unchecked_bet'))) {
    // Ikke testet
    const noOfUnchecked = conv.data.bets.unchecked.length;

    if (noOfUnchecked <= 1) {
      // If there's only one unchecked game
      const uncheckedBet = conv.data.bets.unchecked[0]; // Get the only unchecked bet
      const gameName = uncheckedBet.gameName.split(' ')[0]; // To only get joker if gameName is joker lørdag

      conv.ask(
        responses.help.uncheckedBet(
          utils.formatDayLastWeek(uncheckedBet.draws.draw[0].drawDate),
          gameName
        )
      );
    } else {
      // If there are multiple unchecked games
      conv.ask(responses.help.uncheckedBets(noOfUnchecked));
    }
    conv.contexts.set('unchecked_bet', 2, context.parameters);
  } else if ((context = conv.contexts.get('token_invalid_fnr'))) {
    conv.ask(responses.help.tokenInvalidFnr);
    conv.contexts.set('token_invalid_fnr', 99, context.parameters);
  } else if ((context = conv.contexts.get('token_invalid'))) {
    conv.ask(responses.help.tokenInvalid);
    conv.contexts.set('token_invalid', 99, context.parameters);
  } else if ((context = conv.contexts.get('token_valid'))) {
    conv.ask(responses.help.tokenValid);
    conv.contexts.set('token_valid', 99, context.parameters);
  } else {
    conv.ask(responses.error);
    console.error(
      'Intent infoGeneral was called with none of the contexts set'
    );
  }
});
// Get the user's active bets.
app.intent('getActiveBets', (conv, { resultType }) => {
  bets.getActiveBets(conv, resultType);
});

// If the user input does not trigger any intents
app.intent('Default Fallback Intent', conv => {
  // Allow unknown input 3 times before closing conversation
  if (conv.data.fallbackCount > 2) {
    conv.close(responses.error.closeFallback);
  } else {
    conv.ask(responses.error.listFallback[conv.data.fallbackCount++]);
  }
});

export const dialogflowFirebaseFulfillment = functions
  .region('europe-west1')
  .https.onRequest(app);

export const sendNotificationSMS = functions
  .region('europe-west1')
  .pubsub.topic('sms-notify')
  .onPublish(async () => {
    const collectionRef = db.collection('notifications');

    const querySnapshot = await collectionRef
      .where('date', '<', new Date())
      .get();

    querySnapshot.forEach(doc => {
      const { game, id } = doc.data();
      // const { game } = doc.data();
      const phone = utils.decrypt(doc.data().phone);
      console.log(`Send SMS to ${phone}, game is ${game}`);

      const smsBody = notifications.createNotification(game, id);

      // Send SMS with notification, then log.
      notifications
        .sendSMS(smsBody, phone)
        .then(() =>
          console.log(
            `SMS notification with receipt (id: ${id}) sent to ${phone}.`
          )
        )
        .catch(error =>
          console.error('Error sending SMS notification:', error)
        );

      collectionRef
        .doc(doc.id)
        .delete()
        .then(console.log(`Deleted doc with id ${doc.id}`));
    });
  });
