import * as request from 'request-promise';
import * as constants from './constants';

/**
 * Function to validate that a birth number is 11 digits.
 *
 * @param fnr Birth number
 * @returns {boolean}
 */
export function validateFnr(fnr) {
  const fnrTrimmed = fnr.replace(/[^\d]/g, '');
  const regex = /^[0-3][0-9][0,1][0-9][0-9]{7}$/g;
  const valid = fnrTrimmed.match(regex);

  return valid !== null ? fnrTrimmed : null;
}

/**
 * Function to validate that the one-time password is 4 digits.
 *
 * @param otp One-time password
 * @returns {boolean}
 */
export function validateOTP(otp) {
  const otpTrimmed = otp.replace(/[^\d]/g, '');
  const regex = /^[0-9]{4}$/g;
  const valid = otpTrimmed.match(regex);

  return valid !== null ? otpTrimmed : null;
}

/**
 * Sanitize template literal inputs by escaping characters into XML entities
 * to use in SSML
 * Also normalize the extra spacing for better text rendering in SSML
 * A tag function used by ES6 tagged template literals
 * Sends a POST request to the API to login to Norsk Tipping.
 *
 * @param {string} fnr Birth number
 * @param {request.CookieJar} cookieJar For saving the cookies
 * @returns {Promise}
 */
export function sendPostFnr(fnr, cookieJar) {
  const options = {
    headers: { 'content-type': 'application/json' },
    url: constants.API_URLS.LOGIN,
    body: `{ "personId": "${fnr}"}`,
    jar: cookieJar,
    resolveWithFullResponse: true
  };

  return request.post(options);
}

/**
 * Updates an already existing document in the database.
 *
 * @param {FirebaseFirestore.Firestore} db Reference to the database
 * @param {DialogflowConversation} conv Reference to the document
 * @param {Object} data Data to update in the document
 */
export async function saveDocumentToDB(db, conv, data) {
  try {
    await db
      .collection('users')
      .doc(conv.user.storage.db)
      .update(data);
    console.log('Saved document to database:', conv.user.storage.db);
  } catch (error) {
    return console.error('Error saving document to database:', error);
  }
}

/**
 * Update the token in the database when the user closes the conversation.
 *
 * @param {FirebaseFirestore.Firestore} db Reference to the database
 * @param {DialogflowConversation} conv Reference to the document
 * @param {request.CookieJar} cookieJar Cookie jar
 */
export async function updateTokenOnClose(db, conv, cookieJar) {
  const parsedCookiesFromJar = JSON.parse(JSON.stringify(cookieJar))._jar
    .cookies;

  const sessionID = parsedCookiesFromJar.find(
    cookie => cookie.key === 'JSESSIONID'
  );

  const data = { token: `JSESSIONID=${sessionID.value}` };

  try {
    await db
      .collection('users')
      .doc(conv.user.storage.db)
      .update(data);
    console.log('Updated token in database:', conv.user.storage.db);
  } catch (error) {
    return console.error('Error updating token in database:', error);
  }
}
