import * as request from 'request-promise';
import * as constants from './constants';
import * as utils from './utils';

/**
 * Adds all active bets as notifications to collection 'notifications'. This
 * gets called when the user decides to receive SMS notifications.
 *
 * @param {FirebaseFirestore.Firestore} db Reference to the database
 * @param {request.CookieJar} cookieJar Cookie jar
 */
export async function addAllNotifications(db, cookieJar) {
  const options = {
    url: constants.API_URLS.BETS,
    json: true,
    jar: cookieJar
  };

  // Array for storing active bets.
  let activeBets = [];

  try {
    const response = await request.get(options);

    // Push only active bets to activeBets.
    response.bets.forEach(bet => {
      if (bet.couponActive) activeBets.push(bet);
    });
  } catch (error) {
    return console.error('Error getting bets:', error);
  }

  // Get user's phoneNumber to save in the database.
  const phoneNumber = await utils.getPhone(cookieJar);

  // Add all bets as a notification in the database collection.
  activeBets.forEach(bet => {
    saveNotificationToDB(
      db,
      phoneNumber,
      bet.couponDetails.nextDrawDateUTC,
      bet.gameName,
      bet.saleId
    );
  });
}

/**
 * Deletes all notifications from collection 'notifications'. This gets called
 * when the user decides to not receive any SMS notifications.
 *
 * @param {FirebaseFirestore.Firestore} db Reference to the database
 * @param {request.CookieJar} cookieJar Cookie jar
 */
export async function deleteAllNotifications(db, cookieJar) {
  // Get user's phone number to know which notifications to delete.
  const phoneNumber = await utils.getPhone(cookieJar);

  const collectionRef = db.collection('notifications');

  // Get all notifications with user's phone number.
  const querySnapshot = await collectionRef
    .where('phone', '==', phoneNumber)
    .get();

  // Delete all notifications.
  querySnapshot.forEach(doc => {
    collectionRef
      .doc(doc.id)
      .delete()
      .then(console.log(`Deleted doc with id ${doc.id}`));
  });
}

/**
 * Saves a notification to the database with user's phone number, date of the
 * draw, name of the game and the id of the sale.
 *
 * @param {FirebaseFirestore.Firestore} db Reference to the database
 * @param {number} phoneNumber User's phone number
 * @param {string} lastDrawUTC Draw date of the bet in UTC
 * @param {string} gameName Name of the game
 * @param {string} id
 */
export async function saveNotificationToDB(
  db,
  phoneNumber,
  lastDrawUTC,
  gameName,
  id
) {
  try {
    const docRef = await db.collection('notifications').add({
      phone: phoneNumber,
      date: new Date(lastDrawUTC),
      game: gameName.toUpperCase(),
      id: id
    });

    return console.log(`Added notification with document id ${docRef.id}`);
  } catch (error) {
    return console.error('Error updating document:', error);
  }
}

/**
 * Updates an already existing document in the database.
 *
 * @param {FirebaseFirestore.Firestore} db Reference to the database
 * @param {DialogflowConversation} conv Reference to the document
 * @param {Boolean} sms User wants SMS notifications or not
 */
export async function updateNotificationSetting(db, conv, sms) {
  try {
    const docRef = await db
      .collection('users')
      .doc(conv.user.storage.db)
      .update({ sms: sms });

    return console.log(
      `Updated SMS setting for user with document id ${docRef.id}`
    );
  } catch (error) {
    return console.error('Error updating document:', error);
  }
}

/**
 * Create a body to be used for SMS notification.
 *
 * @param {string} gameName Name of the purchased game
 * @param {string} receiptId ID of the receipt
 */
export function createNotification(gameName, receiptId) {
  const body =
    `Nå er trekningen for ${gameName} ferdig. Snakk med Google Home for å ` +
    'høre hvordan det gikk, eller sjekk kvitteringen her: ' +
    `www.norsk-tipping.no/kvittering?${receiptId}.`;

  return body;
}

/**
 * Create a body to be used for SMS receipt.
 *
 * @param {string} gameName Name of the purchased game
 * @param {number} couponPrice Price of the purchased game
 * @param {string} receiptId ID of the receipt
 * @param {string} drawDate Draw date of the purchased game
 */
export function createSMS(gameName, couponPrice, receiptId, drawDate) {
  const body =
    `Kvittering ${receiptId}.\nDu har levert ${gameName.toUpperCase()} med ` +
    `trekning ${utils.formatDate(drawDate)}. Kr ${couponPrice},- ble trukket ` +
    'fra din konto. Klikk her for å se dine tall: ' +
    `www.norsk-tipping.no/kvittering?${receiptId}`;

  return body;
}

/**
 * Send an SMS with a receipt of a game to the specified phone number.
 *
 * @param {string} body Body for the SMS
 * @param {string} number Receiever's phone number
 * @returns {Promise}
 */
export function sendSMS(body, number) {
  return request.post({
    url: constants.SMS.URL,
    oauth: {
      consumer_key: constants.SMS.CONSUMER_KEY,
      consumer_secret: constants.SMS.CONSUMER_SECRET
    },
    json: true,
    body: {
      sender: constants.SMS.SENDER,
      message: body,
      recipients: [{ msisdn: number }]
    }
  });
}
