import * as functions from 'firebase-functions';
import * as request from 'request-promise';
import * as constants from './constants';
import { Crypto } from 'cryptojs';

/**
 * Sends a GET request to the API to check if the user is authenticated with the
 * current JSESSIONID and get the name of the user.
 *
 * @param {CookieJar} cookieJar Cookie jar
 * @returns {Promise}
 */
export async function getName(cookieJar) {
  const options = {
    url: constants.API_URLS.INFO,
    json: true,
    jar: cookieJar
  };

  try {
    const response = await request.get(options);
    const name = response.firstName.replace(
      /\w\S*/g,
      name => name.charAt(0).toUpperCase() + name.substr(1).toLowerCase()
    );
    return ' ' + name;
  } catch (error) {
    console.error(error);
    return '';
  }
}

/**
 * Sends a GET request to the API to get the user's phone number.
 *
 * @param {CookieJar} cookieJar Cookie jar
 * @returns {Promise}
 */
export async function getPhone(cookieJar) {
  const options = {
    url: constants.API_URLS.INFO,
    json: true,
    jar: cookieJar
  };

  try {
    const response = await request.get(options);
    return response.contactInfo.phoneMobile;
  } catch (error) {
    return console.error(error);
  }
}

/**
 * Extracts the JSESSIONID from the response headers.
 *
 * @param response Request response
 * @returns {string}
 */
export function getCookie(response) {
  const cookies = response.headers['set-cookie'];
  // TODO: Find a clean way to extract the cookie. The
  // compiler complains if strict is set to true, due to
  // "Object possibly undefined".
  const sessionID = cookies
    .find(cookie => cookie.startsWith('JSESSIONID'))
    .split(';')[0];

  return sessionID;
}

/**
 * Format template literal inputs to a single line.
 *
 * @example
 * const equation = '"1 + 1 > 1"';
 * const response = ssml`
 *   <speak>
 *     ${equation}
 *   </speak>
 * `;
 * // Equivalent to ssml`\n  <speak>\n    ${equation}\n  </speak>\n`
 * console.log(response);
 *
 * Prints:
 * `<speak>"1 + 1 > 1"</speak>`
 *
 * @param {TemplateStringsArray} template Non sanitized constant strings in the
 *    template literal
 * @param {Array<string>} inputs Computed expressions to be sanitized surrounded
 *    by ${}
 * @returns {string}
 */
export function ssml(template, ...inputs) {
  // Generate the raw escaped string
  const raw = template.reduce((out, str, i) =>
    i ? out + inputs[i - 1] + str : str
  );

  // Trim out new lines at the start and end but keep indentation
  const trimmed = raw
    .replace(/^\s*\n(\s*)<speak>/, '$1<speak>')
    .replace(/<\/speak>\s+$/, '</speak>');

  // Split lines on new line
  const lines = trimmed.split('\n');
  // Trim the lines and join them with a white space
  return lines
    .map(line => line.trim())
    .join(' ')
    .replace(/>\s</g, '><');
}

/**
 * Get the balance on the player account
 *
 * @param {CookieJar} cookieJar Cookie jar
 */
export async function getBalance(cookieJar) {
  // Create JSON payload
  const options = {
    url: constants.API_URLS.BALANCE,
    json: true,
    jar: cookieJar
  };

  try {
    const response = await request.get(options);
    return response.balance;
  } catch (error) {
    console.error('Error getting balance:', error);
    return null;
  }
}

/**
 * Format a date to norwegian locale.
 *
 * @param {string} drawDate UTC date string
 * @returns {string} Formatted date
 */
export function formatDate(drawDate) {
  const months = [
    'januar',
    'februar',
    'mars',
    'april',
    'mai',
    'juni',
    'juli',
    'august',
    'september',
    'oktober',
    'november',
    'desember'
  ];

  const date = new Date(drawDate);

  return `${date.getDate()} ${months[date.getMonth()]} ${date.getFullYear()}`;
}

/**
 * Formats a day within the last seven days as "i dag" if today,
 * "i går" if it was yesterday, "sist <weekDay>" if seven days ago,
 * otherwise "på <weekDay>"
 * @param {drawDate} drawDate UTC date string
 */
export function formatDayLastWeek(drawDate) {
  const weekDays = [
    'søndag',
    'mandag',
    'tirsdag',
    'onsdag',
    'torsdag',
    'fredag',
    'lørdag'
  ];
  const dateDrawn = new Date(drawDate);
  const daysSince = Math.floor(
    (new Date() - dateDrawn) / (1000 * 60 * 60 * 24)
  );

  if (daysSince > 7) {
    console.warn(
      'formatDayLastWeek() called with a date more than seven days ago'
    );
  }

  switch (daysSince) {
  case 0:
    return 'i dag';
  case 1:
    return 'i går';
  case 7:
    return 'sist ' + weekDays[dateDrawn.getUTCDay()];
  }

  return 'på ' + weekDays[dateDrawn.getUTCDay()];
}

export function encrypt(string) {
  return Crypto.AES.encrypt(string, functions.config().crypto.secret);
}

export function decrypt(encrypted) {
  return Crypto.AES.decrypt(encrypted, functions.config().crypto.secret);
}
